-- -*-lua-*-

require('saotrace.log').init( 'testlog.log' )

local create = require('saotrace.baffles.ideal').create
local table = require( 'table' )
local os = require( 'os' )
local srcdir = os.getenv('srcdir') or '.'
local va = require('validate.args')

local geometry_db = table.concat( { srcdir, 'data', 'geometry.rdb'}, '/' )
local one_entry_db = table.concat( { srcdir, 'data', 'one-entry.rdb'}, '/' )


local giter = require('saotrace.geometry').read{ file = geometry_db, mode = 'iterator' }

local optic = {}

local cols = {
   x = 'x0',
   y = 'y0',
   z = 'z_f',
   r = 'rho_f',
}

-- read in geometry file so we can check that things are read in
-- properly
for geo in giter do

   if not optic[geo.optic] then
      optic[geo.optic] = {}
   end

   optic[geo.optic][geo.shell] = geo

end

describe( "create ideal entrance aperture", function()

    setup( function()
	     va.reset()
             va.opts{ check_spec = true,
		      error_on_bad_spec = true,
	            }
           end
    )

    describe( "outermost", function()

	local inner, outer =
	create{ shell = 1,
	    optic = 'p',
	    cols = cols,
	    db = geometry_db
	}

	-- test for correct radii of generated circles.

	it( "p3 is the inner shell", function()
	    assert.are.equal( optic.p[3].rho_f, inner.r )
         end)


	it( "p1 is the outer shell", function()
	      assert.are.equal( optic.p[1].rho_f, outer.r )
	end)

     end)

    describe( "middle", function()

	local inner, outer =
	create{ shell = 3,
		optic = 'p',
		cols = cols,
		db = geometry_db
	     }

	-- test for correct radii of generated circles.

	it( "p4 is the inner shell", function()
	    assert.are.equal( optic.p[4].rho_f, inner.r )
         end)


	it( "p3 is the outer shell", function()
	      assert.are.equal( optic.p[3].rho_f, outer.r )
	end)

     end)


    describe( "inner disk", function()

	local inner, outer =
	   create{ shell = 6,
	       optic = 'p',
	       cols = cols,
	       db = geometry_db,
	       inner_disk = { z = 20,
			      r = 10 },
	    }

	it( "p4-p6 z", function()
            assert.are.equal( 20, inner.z )
	 end)

	it( "p4-p6 r", function()
            assert.are.equal( 10, inner.r )
	 end)

	it( "p6 is the outer shell", function()
	      assert.are.equal( optic.p[6].rho_f, outer.r )
	end)

    end)

    describe( "no inner disk", function()

	local inner, outer =
	create{ shell = 6,
	       optic = 'p',
	       cols = cols,
	       db = geometry_db
	    }

        it( "p4-p6 r", function()
            assert.are.equal( optic.p[6].rho_f - (optic.p[4].rho_f - optic.p[6].rho_f),
		 inner.r )
	 end)

	it( "p6 is the outer shell", function()
	      assert.are.equal( optic.p[6].rho_f, outer.r )
	end)
     end)
 end)


describe( "test errors", function()

    setup( function()
	     va.reset()
             va.opts{ check_spec = true,
		      error_on_bad_spec = true,
	            }
           end
    )

    describe( "too few elements", function()

        it( "p1", function()
            assert.has.error( function()
				  create{ shell = 1,
					  optic = 'p',
					  cols = cols,
					  db = one_entry_db
				       }
			       end,
			     'could not find neighboring mirror'
			   )
	 end)

     end)

    describe( "mirror not found ", function()

        it( "p1", function()
            assert.has.error( function()
				  create{ shell = 8,
					  optic = 'p',
					  cols = cols,
					  db = one_entry_db
				       }
			       end,
			     'unable to find mirror'
			   )
	 end)

     end)


 end)
